@@ -260,7 +260,11 @@ sub AUTOLOAD {
 ##################################################
 sub DESTROY {
 ##################################################
-    # just there because of AUTOLOAD
+    warn "Destroying appender $_[0]" if $Log::Log4perl::CHATTY_DESTROY_METHODS;
+
+    foreach my $key (keys %{$_[0]}) {
+        delete $_[0]->{$key};
+    }
 }
 
 1;
