@@ -28,6 +28,14 @@ sub new {
     return $self;
 }
 
+##################################################
+sub filename {
+##################################################
+    my($self) = @_;
+
+    return $self->{filename};
+}
+
 ##################################################
 sub file_open {
 ##################################################
