@@ -10,7 +10,7 @@ package Log::Log4perl::Config::DOMConfigurator;
 # DONEsee Config, need to check version of XML::DOM
 # OK user defined levels? see parse_level
 # OK make sure 2nd test is using log4perl constructs, not log4j
-# handle new filter stuff
+# OK handle new filter stuff
 # make sure sample code actually works
 # try removing namespace prefixes in the xml
 
@@ -22,7 +22,10 @@ use constant _INTERNAL_DEBUG => 0;
 
 our $VERSION = 0.03;
 
-our $APPENDER_TAG = qr/((log4j|log4perl):)?appender/;
+our $APPENDER_TAG = qr/^((log4j|log4perl):)?appender$/;
+
+our $FILTER_TAG = qr/^(log4perl:)?filter$/;
+our $FILTER_REF_TAG = qr/^(log4perl:)?filter-ref$/;
 
 #can't use ValParser here because we're using namespaces? 
 #doesn't seem to work - kg 3/2003 
@@ -50,12 +53,12 @@ sub parse {
     my $config = $doc->getElementsByTagName("$LOG4J_PREFIX:configuration")->item(0)||
                  $doc->getElementsByTagName("$LOG4PERL_PREFIX:configuration")->item(0);
 
-    my $threshold = uc($config->getAttribute('threshold'));
+    my $threshold = uc(subst($config->getAttribute('threshold')));
     if ($threshold) {
         $l4p_tree->{threshold}{value} = $threshold;
     }
 
-    if ($config->getAttribute('oneMessagePerAppender') eq 'true') {
+    if (subst($config->getAttribute('oneMessagePerAppender')) eq 'true') {
         $l4p_tree->{oneMessagePerAppender}{value} = 1;
     }
 
@@ -78,6 +81,10 @@ sub parse {
         }elsif ($tag_name eq 'root'){
             &parse_root($l4p_tree, $kid);
 
+        }elsif ($tag_name =~ $FILTER_TAG){
+            #parse log4perl's chainable boolean filters
+            &parse_l4p_filter($l4p_tree, $kid);
+
         }elsif ($tag_name eq 'renderer'){
             warn "Log4perl: ignoring renderer tag in config, unimplemented";
             #"log4j will render the content of the log message according to 
@@ -105,7 +112,7 @@ sub parse_patternlayout {
     for my $child ($node->getChildNodes) {
         next unless $child->getNodeType == ELEMENT_NODE;
 
-        my $name = $child->getAttribute('name');
+        my $name = subst($child->getAttribute('name'));
         my $value;
 
         foreach my $grandkid ($child->getChildNodes){
@@ -115,7 +122,7 @@ sub parse_patternlayout {
         }
         $value =~ s/^ +//;  #just to make the unit tests pass
         $value =~ s/ +$//;
-        $l4p_branch->{$name}{value} = $value;
+        $l4p_branch->{$name}{value} = subst($value);
     }
     $l4p_tree->{PatternLayout}{cspec} = $l4p_branch;
 }
@@ -133,12 +140,56 @@ sub parse_root {
 
 }
 
+
+#this parses a custom log4perl-specific filter set up under
+#the root element, as opposed to children of the appenders
+sub parse_l4p_filter {
+    my ($l4p_tree, $node) = @_;
+
+    my $l4p_branch = {};
+
+    my $name = subst($node->getAttribute('name'));
+
+    my $class = subst($node->getAttribute('class'));
+    my $value = subst($node->getAttribute('value'));
+
+    if ($class && $value) {
+        die "Log4perl: only one of class or value allowed, not both, "
+            ."in XMLConfig filter '$name'";
+    }elsif ($class || $value){
+        $l4p_branch->{value} = ($value || $class);
+
+    }
+
+    for my $child ($node->getChildNodes) {
+
+        if ($child->getNodeType == ELEMENT_NODE){
+
+            my $tag_name = $child->getTagName();
+
+            if ($tag_name =~ /^(param|param-nested|param-text)$/) {
+                &parse_any_param($l4p_branch, $child);
+            }
+        }elsif ($child->getNodeType == TEXT_NODE){
+            my $text = $child->getData;
+            next unless $text =~ /\S/;
+            if ($class && $value) {
+                die "Log4perl: only one of class, value or PCDATA allowed, "
+                    ."in XMLConfig filter '$name'";
+            }
+            $l4p_branch->{value} .= subst($text); 
+        }
+    }
+
+    $l4p_tree->{filter}{$name} = $l4p_branch;
+}
+
    
 #for parsing a category/logger element
 sub parse_category {
     my ($l4p_tree, $node) = @_;
 
-    my $name = $node->getAttribute('name');
+    my $name = subst($node->getAttribute('name'));
 
     $l4p_tree->{category} ||= {};
  
@@ -151,14 +202,14 @@ sub parse_category {
 
     my $l4p_branch = $ptr;
 
-    my $class = $node->getAttribute('class');
+    my $class = subst($node->getAttribute('class'));
     $class                       && 
        $class ne 'Log::Log4perl' &&
        $class ne 'org.apache.log4j.Logger' &&
        warn "setting category $name to class $class ignored, only Log::Log4perl implemented";
 
     #this is kind of funky, additivity has its own spot in the tree
-    my $additivity = $node->getAttribute('additivity');
+    my $additivity = subst(subst($node->getAttribute('additivity')));
     if (length $additivity > 0) {
         $l4p_tree->{additivity} ||= {};
         my $add_ptr = $l4p_tree->{additivity};
@@ -185,15 +236,15 @@ sub parse_children_of_logger_element {
         my $tag_name = $child->getTagName();
 
         if ($tag_name eq 'param') {
-            my $name = $child->getAttribute('name');
-            my $value = $child->getAttribute('value');
+            my $name = subst($child->getAttribute('name'));
+            my $value = subst($child->getAttribute('value'));
             if ($value =~ /^(all|debug|info|warn|error|fatal|off|null)^/) {
                 $value = uc $value;
             }
             $l4p_branch->{$name} = {value => $value};
         
         }elsif ($tag_name eq 'appender-ref'){
-            push @appenders, $child->getAttribute('ref');
+            push @appenders, subst($child->getAttribute('ref'));
             
         }elsif ($tag_name eq 'level' || $tag_name eq 'priority'){
             $priority = &parse_level($child);
@@ -208,7 +259,7 @@ sub parse_children_of_logger_element {
 sub parse_level {
     my $node = shift;
 
-    my $level = uc ($node->getAttribute('value'));
+    my $level = uc (subst($node->getAttribute('value')));
 
     die "Log4perl: invalid level in config: $level"
         unless Log::Log4perl::Level::is_valid($level);
@@ -221,11 +272,11 @@ sub parse_level {
 sub parse_appender {
     my ($l4p_tree, $node) = @_;
 
-    my $name = $node->getAttribute("name");
+    my $name = subst($node->getAttribute("name"));
 
     my $l4p_branch = {};
 
-    my $class = $node->getAttribute("class");
+    my $class = subst($node->getAttribute("class"));
 
     $l4p_branch->{value} = $class;
 
@@ -236,8 +287,7 @@ sub parse_appender {
 
         my $tag_name = $child->getTagName();
 
-        my $name = unlog4j($child->getAttribute('name'));
-
+        my $name = unlog4j(subst($child->getAttribute('name')));
 
         if ($tag_name =~ /^(param|param-nested|param-text)$/) {
 
@@ -248,8 +298,11 @@ sub parse_appender {
         }elsif ($tag_name =~ /($LOG4PERL_PREFIX:)?layout/){
             $l4p_branch->{layout} = parse_layout($child);
 
-        }elsif ($tag_name eq 'filter'){
-            die "filters not supported yet";
+        }elsif ($tag_name =~  $FILTER_TAG){
+            $l4p_branch->{Filter} = parse_filter($child);
+
+        }elsif ($tag_name =~ $FILTER_REF_TAG){
+            $l4p_branch->{Filter} = parse_filter_ref($child);
 
         }elsif ($tag_name eq 'errorHandler'){
             die "errorHandlers not supported yet";
@@ -262,17 +315,18 @@ sub parse_appender {
             #I don't see the need to support this on the perl side any 
             #time soon.  --kg 3/2003
             die "Log4perl: in config file, <appender-ref> tag is unsupported in <appender>";
+        }else{
+            die "Log4perl: in config file, <$tag_name> is unsupported\n";
         }
     }
     $l4p_tree->{appender}{$name} = $l4p_branch;
 }
 
-
 sub parse_any_param {
     my ($l4p_branch, $child) = @_;
 
     my $tag_name = $child->getTagName();
-    my $name = $child->getAttribute('name');
+    my $name = subst($child->getAttribute('name'));
     my $value;
 
     print "parse_any_param: <$tag_name name=$name\n" if _INTERNAL_DEBUG;
@@ -292,7 +346,7 @@ sub parse_any_param {
     #<param>
     }elsif ($tag_name eq 'param') {
 
-         $value = $child->getAttribute('value');
+         $value = subst($child->getAttribute('value'));
 
          print "parse_param_nested: got param $name = $value\n"  
              if _INTERNAL_DEBUG;
@@ -318,7 +372,8 @@ sub parse_any_param {
             $value = eval_if_perl($value);
         }
     }
-    
+
+    $value = subst($value);
 
      #multiple values for the same param name
      if (defined $l4p_branch->{$name}{value} ) {
@@ -351,13 +406,55 @@ sub parse_param_nested {
     return $l4p_branch;
 }
 
+#this handles filters that are children of appenders, as opposed
+#to the custom filters that go under the root element
+sub parse_filter {
+    my $node = shift;
+
+    my $filter_tree = {};
+
+    my $class_name = subst($node->getAttribute('class'));
+
+    $filter_tree->{value} = $class_name;
+
+    print "\tparsing filter on class $class_name\n"  if _INTERNAL_DEBUG;  
+
+    for my $child ($node->getChildNodes) {
+        next unless $child->getNodeType == ELEMENT_NODE;
+
+        my $tag_name = $child->getTagName();
+
+        if ($tag_name =~ 'param|param-nested|param-text') {
+            &parse_any_param($filter_tree, $child);
+        
+        }else{
+            die "Log4perl: don't know what to do with a ".$child->getTagName()
+                ."inside a filter element";
+        }
+    }
+    return $filter_tree;
+}
+
+sub parse_filter_ref {
+    my $node = shift;
+
+    my $filter_tree = {};
+
+    my $filter_id = subst($node->getAttribute('id'));
+
+    $filter_tree->{value} = $filter_id;
+
+    return $filter_tree;
+}
+
+
 
 sub parse_layout {
     my $node = shift;
 
     my $layout_tree = {};
 
-    my $class_name = $node->getAttribute('class');
+    my $class_name = subst($node->getAttribute('class'));
     
     $layout_tree->{value} = $class_name;
     #
@@ -365,8 +462,8 @@ sub parse_layout {
     for my $child ($node->getChildNodes) {
         next unless $child->getNodeType == ELEMENT_NODE;
         if ($child->getTagName() eq 'param') {
-            my $name = $child->getAttribute('name');
-            my $value = $child->getAttribute('value');
+            my $name = subst($child->getAttribute('name'));
+            my $value = subst($child->getAttribute('value'));
             if ($value =~ /^(all|debug|info|warn|error|fatal|off|null)$/) {
                 $value = uc $value;
             }
@@ -375,7 +472,7 @@ sub parse_layout {
             $layout_tree->{$name}{value} = $value;  
 
         }elsif ($child->getTagName() eq 'cspec') {
-            my $name = $child->getAttribute('name');
+            my $name = subst($child->getAttribute('name'));
             my $value;
             foreach my $grandkid ($child->getChildNodes){
                 if ($grandkid->getNodeType == TEXT_NODE) {
@@ -384,7 +481,7 @@ sub parse_layout {
             }
             $value =~ s/^ +//;
             $value =~ s/ +$//;
-            $layout_tree->{cspec}{$name}{value} = $value;  
+            $layout_tree->{cspec}{$name}{value} = subst($value);  
         }
     }
     return $layout_tree;
@@ -402,6 +499,16 @@ sub parse_boolean {
     }
 }
 
+
+#this handles variable substitution
+sub subst {
+    my $val = shift;
+
+    $val =~ s/\${(.*?)}/
+                      Log::Log4perl::Config::var_subst($1, {})/gex;
+    return $val;
+}
+
 1;
 
 __END__
@@ -606,7 +713,52 @@ The features added by the log4perl dtd are:
      log4j.appender.jabbender.login.port     = 5222
      log4j.appender.jabbender.login.username = bobjones
 
-=item 5 the new filter stuff!! (TBD)
+=item 5 the log4perl-specific filters, see L<Log::Log4perl::Filter>,
+lots of examples in t/044XML-Filter.t, here's a short one:
+
+
+  <?xml version="1.0" encoding="UTF-8"?> 
+  <!DOCTYPE log4perl:configuration SYSTEM "log4perl.dtd">
+
+  <log4perl:configuration xmlns:log4perl="http://log4perl.sourceforge.net/">
+   
+  <appender name="A1" class="Log::Log4perl::Appender::TestBuffer">
+        <layout class="Log::Log4perl::Layout::SimpleLayout"/>
+        <filter class="Log::Log4perl::Filter::Boolean">
+            <param name="logic" value="!Match3 &amp;&amp; (Match1 || Match2)"/> 
+        </filter>
+  </appender>   
+  
+  <appender name="A2" class="Log::Log4perl::Appender::TestBuffer">
+        <layout class="Log::Log4perl::Layout::SimpleLayout"/>
+        <filter-ref id="Match1"/>
+  </appender>   
+  
+  <log4perl:filter name="Match1" value="sub { /let this through/ }" />
+  
+  <log4perl:filter name="Match2">
+        sub { 
+            /and that, too/ 
+        }
+   </log4perl:filter>
+  
+  <log4perl:filter name="Match3" class="Log::Log4perl::Filter::StringMatch">
+    <param name="StringToMatch" value="suppress"/>
+    <param name="AcceptOnMatch" value="true"/>
+  </log4perl:filter>
+  
+  <log4perl:filter name="MyBoolean" class="Log::Log4perl::Filter::Boolean">
+    <param name="logic" value="!Match3 &amp;&amp; (Match1 || Match2)"/>
+  </log4perl:filter>
+  
+   
+   <root>
+           <priority value="info"/>
+           <appender-ref ref="A1"/>
+   </root>
+   
+   </log4perl:configuration>
+
 
 =back
 
@@ -669,6 +821,16 @@ instead of just <appender>, you can make your own DTD combining
 the two DTDs and getting rid of the namespace prefixes.  Then you can
 validate against that, and log4perl should accept it just fine.
 
+=head1 VARIABLE SUBSTITUTION
+
+This supports variable substitution like C<${foobar}> in text and in 
+attribute values except for appender-ref.  If an environment variable is defined
+for that name, its value is substituted. So you can do stuff like
+
+        <param name="${hostname}" value="${hostnameval}.foo.com"/>
+        <param-text name="to">${currentsysadmin}@foo.com</param-text>
+
+
 =head1 REQUIRES
 
 To use this module you need XML::DOM installed.  
