@@ -5,9 +5,10 @@ use strict;
 
 package Log::Log4perl::Config::Watch;
 
-use constant DEBUG => 0;
+use constant _INTERNAL_DEBUG => 0;
 
 our $NEXT_CHECK_TIME;
+our $SIGNAL_CAUGHT;
 
 ###########################################
 sub new {
@@ -16,6 +17,7 @@ sub new {
 
     my $self = { file            => "",
                  check_interval  => 30,
+                 signal          => undef,
                  %options,
                  _last_checked_at => 0,
                  _last_timestamp  => 0,
@@ -23,8 +25,15 @@ sub new {
 
     bless $self, $class;
 
-        # Just called to initialize
-    $self->change_detected();
+    if($self->{signal}) {
+            # We're in signal mode, set up the handler
+        $SIG{$self->{signal}} = sub { $SIGNAL_CAUGHT = 1; };
+            # Reset the marker. The handler is going to modify it.
+        $SIGNAL_CAUGHT = 0;
+    } else {
+            # Just called to initialize
+        $self->change_detected();
+    }
 
     return $self;
 }
@@ -37,6 +46,14 @@ sub file {
     return $self->{file};
 }
 
+###########################################
+sub signal {
+###########################################
+    my($self) = @_;
+
+    return $self->{signal};
+}
+
 ###########################################
 sub check_interval {
 ###########################################
@@ -52,12 +69,12 @@ sub change_detected {
 
     $time = time() unless defined $time;
 
-    print "Calling change_detected (time=$time)\n" if DEBUG;
+    print "Calling change_detected (time=$time)\n" if _INTERNAL_DEBUG;
 
         # Do we need to check?
     if($self->{_last_checked_at} + 
        $self->{check_interval} > $time) {
-        print "No need to check\n" if DEBUG;
+        print "No need to check\n" if _INTERNAL_DEBUG;
         return ""; # don't need to check, return false
     }
        
@@ -70,12 +87,12 @@ sub change_detected {
 
     if($new_timestamp > $self->{_last_timestamp}) {
         $self->{_last_timestamp} = $new_timestamp;
-        print "Change detected (store=$new_timestamp)!\n" if DEBUG;
+        print "Change detected (store=$new_timestamp)!\n" if _INTERNAL_DEBUG;
         return 1; # Has changed
     }
        
     print "Hasn't changed (file=$new_timestamp ",
-          "stored=$self->{_last_timestamp})!\n" if DEBUG;
+          "stored=$self->{_last_timestamp})!\n" if _INTERNAL_DEBUG;
     return "";  # Hasn't changed
 }
 
@@ -126,6 +143,8 @@ have elapsed since the last physical check. If the file's modification
 date has changed since the last physical check, it will return a true 
 value, otherwise a false value is returned.
 
+=back
+
 Bottom line: C<check_interval> allows you to call the function
 C<change_detected()> as often as you like, without paying the performing
 a significant performance penalty because file system operations 
@@ -149,7 +168,24 @@ on to C<change_detected()> as an optional parameter, like in
 which then won't trigger a call to C<time()>, but use the value
 provided.
 
-=back
+=head2 SIGNAL MODE
+
+Instead of polling time and file changes, C<new()> can be instructed 
+to set up a signal handler. If you call the constructor like
+
+    my $watcher = Log::Log4perl::Config::Watch->new(
+                          file    => "/data/my.conf",
+                          signal  => 'HUP'
+                  );
+
+then a signal handler like
+
+    $SIG{HUP} = sub { $SIGNAL_CAUGHT = 1; };
+
+will be installed, setting a globally accessible variable 
+C<Log::Log4perl::Config::Watch::SIGNAL_CAUGHT> to a true value when
+the signal arrives. Comes with all the problems that signal handlers
+go along with.
 
 =head1 SEE ALSO
 
